use std::rc::Rc;
use futures::executor::block_on;
use gtk4::{prelude::*};
use gtk4::subclass::prelude::*;
use gtk4::{
    gdk::Paintable,
    glib::*,
    ApplicationWindow,
    HeaderBar,
    Button,
    Window,
    Popover,
    Builder,
    IconTheme,
    IconLookupFlags,
    TextDirection,
    MenuButton,
};

use crate::popover::CustomPopoverMenu;

#[derive(Debug, Downgrade)]
pub struct CustomTitleBar {
    pub window: Window,
    pub titlebar: HeaderBar,
    pub run_button: Button,
    pub menu_button: MenuButton,
    pub popover_menu: CustomPopoverMenu,
}

impl CustomTitleBar {
    pub fn new(pop: &CustomPopoverMenu) -> CustomTitleBar {
        // loading PopoverMenu logic goes here
        CustomTitleBar {
            window: Window::new(),
            titlebar: HeaderBar::new(),
            run_button: Button::new(),
            menu_button: MenuButton::new(),
            popover_menu: pop.clone(),
        }
    }

    pub fn set_buttons(&self) {
        // set labels
        self.run_button.set_label("Apply Patch");
        //self.menu_button.set_label("Menu");

        // set css class; it turns blue!
        self.run_button.set_css_classes(&["run_btn"]);

        // set visibility
        self.run_button.set_visible(true);
        self.menu_button.set_visible(true);

        // symbolic
        // let menu_button = IconTheme::new();
        // menu_button.lookup_icon("open-menu-symbolic", &[], 16, 1, TextDirection::Ltr, IconLookupFlags::empty());
        self.menu_button.set_popover(Some(&self.popover_menu.popover));

    }

    pub fn click_run_button(&self) {
        self.run_button.connect_clicked(move |button| {
            button.set_label("Avery is awesome!");
        });
    }

    pub fn click_menu(&self) {
        // self.menu_button.connect_clicked(clone!(@weak self as this =>
        //     move |_| {
        //         this.popover.show();
        // }));
    }

    pub fn set_window(&self, window: &ApplicationWindow) {
        window.set_titlebar(Some(&self.titlebar));
    }

    pub fn set_titlebar(&self) {
        self.titlebar.pack_start(&self.run_button);
        self.titlebar.pack_end(&self.menu_button);
        self.titlebar.set_css_classes(&["headerbar"])
    }

    pub fn set_popover(&self) {
        // huh? why is this method here again?
    }
}